from datetime import datetime, timedelta

from pyPhases.util.Logger import classLogger
from pyPhasesRecordloader import Event


@classLogger
class DominoAnnotationLoader:
    headers = {
        "Signal ID": "id",
        "Start Time": "startTime",
        "Signal Type": "signalType",
        "Events list": "possibleValues",
        "Rate": "Rate",
        "Unit": "unit",
    }

    def __init__(self):
        self.state = "meta"

        self.id = ""
        self.startTime = None
        self.unit = None
        self.signalType = None
        self.valueMap = {
            'N4': 'N3'
        }
        self.possibleValues = None

        self.targetFrequency = 1
        self.events = []
        self.curOffset = 0
        self.curValue = None
        self.eventStart = 0
        self.distinct = True
        self.lastDistinct = None

    def wrapTime(self, timeInSeconds):
        return int(timeInSeconds) * self.targetFrequency
    
    def addEvent(self, start, end, value):
        duration = end - start
        value = self.valueMap[value] if self.valueMap is not None and value in self.valueMap else value
        self.events.append(Event(name=value, start=start, duration=duration))

    def finishEvent(self):
        if self.curValue is not None and self.curValue != "A":
            start = self.wrapTime(self.eventStart)
            value = self.curValue
            if self.distinct:
                if self.lastDistinct is not None:
                    self.lastDistinct.duration = start - self.lastDistinct.start
                self.addEvent(start, start, value)
                self.lastDistinct = self.events[-1]
            else:
                end = self.wrapTime(self.curOffset)
                self.addEvent(start, end, value)

    def addContentLine(self, line):
        values = [v.strip() for v in line.split(";")]

        if self.signalType == "Discret":
            time, value = values
            # time = "%i.%i.%i "%(self.startTime.)
            # time = datetime.strptime(value, "%d.%m.%Y %H:%M:%S")
            if self.curValue is None or self.curValue != value:
                self.finishEvent()
                self.eventStart = self.curOffset

            self.curValue = value
            self.curOffset += 30
        elif self.signalType == "Impuls":
            time, duration, value = values
            time = datetime.strptime(time.split("-")[0], "%H:%M:%S,%f")
            time = datetime.combine(self.startTime, time.time())
            startOffset = time - self.startTime
            # add a day if its beyond 24h
            if startOffset < timedelta(0):
                startOffset = startOffset + timedelta(days=1)
            
            startOffset = startOffset.total_seconds()
            startOffset = self.wrapTime(startOffset)
            duration = float(duration)

            self.addEvent(startOffset, startOffset+duration, value)

    def validateMeta(self):
        assert self.id is not None
        assert self.startTime is not None
        assert self.possibleValues is not None
        assert self.signalType is not None
        assert self.signalType in ["Discret", "Impuls"], "at the moment only Discret/Impulse signalTypes are supported"

        if self.id == "SchlafProfil\profil":
            assert self.Rate == "30 s", f"at the moment only 30 sec Schlafprofil is supported, failed value {self.Rate}"
            assert self.possibleValues in [["N3", "N2", "N1", "Rem", "Wach", "Artefakt"], ['N4', 'N3', 'N2', 'N1', 'Rem', 'Wach', 'Bewegung']], f"only AASM Schlafprofil is supported, failed value: {self.possibleValues}"

    def parseMetaLine(self, line):
        type, value = [v.strip() for v in line.split(":", 1)]
        type = self.headers[type]

        if type == "startTime":
            try:
                value = datetime.strptime(value, "%d.%m.%Y %H:%M:%S")  # 19.11.2020 19:30:30
            except ValueError:
                try:
                    value = datetime.strptime(value, "%d.%m.%Y")  # 19.11.2020
                except ValueError:
                    value = None

        elif type == "possibleValues":
            value = [v.strip() for v in value.split(",")]

        self.__setattr__(type, value)

    def parseLine(self, line):
        if line == "":
            self.state = "content"
            self.validateMeta()
        elif self.state == "meta":
            self.parseMetaLine(line)
        else:
            self.addContentLine(line)

    def parseLines(self, lines):
        for line in lines:
            self.parseLine(line.strip())
        self.finishEvent()

    @staticmethod
    def load(path, valueMap=None, targetFrequency=1, possibleValues=None):
        dl = DominoAnnotationLoader()
        dl.valueMap = valueMap
        dl.targetFrequency = targetFrequency
        dl.possibleValues = possibleValues

        with open(path, "r", encoding="iso-8859-1") as f:
            content = f.readlines()

        dl.parseLines(content)

        return dl



import csv

from pyPhases.util.Logger import classLogger


@classLogger
class DominoErgLoader:
    txtEncoding = "utf-16"

    def getTimeValue(self, timeValue, unit="m"):
        # in minutes to make it compatible with Alice
        h, m, s = timeValue.split(":")
        value = int(s) + int(m) * 60 + int(h) * 60 * 60
        if unit == "m":
            value /= 60
        elif unit == "h":
            value /= 60
        elif unit == "s":
            pass
        else:
            raise Exception("not a valid time unit")
        return value

    def calculateMetaData(self, allValues):
        # default
        supineSleepDuration = (self.getTimeValue(allValues["SUPINE_SLEEP_DURATION"]))
        nonSupineSleepDuration = self.getTimeValue(allValues["PRONE_SLEEP_DURATION"]) + self.getTimeValue(allValues["LEFT_SLEEP_DURATION"]) +self.getTimeValue(allValues["RIGHT_SLEEP_DURATION"])
        tibDuration = self.getTimeValue(allValues["TIB_DURATION"])
        remDuration = self.getTimeValue(allValues["REM_DURATION"])
        nonRemDuration = self.getTimeValue(allValues["NONREM_DURATION"])
        apneaHyponea =float(allValues["APNOEA_SLEEP_NUMBER"])+                    float(allValues["HYPOPNOEA_SLEEP_NUMBER"])
        data = {
            "ahiRuecken": ((float(allValues["APNOEA_SUPINE_NUMBER"]) + float(allValues["HYPOPNOEA_SUPINE_NUMBER"])) / supineSleepDuration) if supineSleepDuration > 0 else 0,
            "ahiKeinRuecken": ((
                    float(allValues["APNOEA_PRONE_NUMBER"]) + 
                    float(allValues["APNOEA_LEFT_NUMBER"]) + 
                    float(allValues["APNOEA_RIGHT_NUMBER"]) + 
                    float(allValues["HYPOPNOEA_PRONE_NUMBER"]) +
                    float(allValues["HYPOPNOEA_LEFT_NUMBER"]) +
                    float(allValues["HYPOPNOEA_RIGHT_NUMBER"])
                ) / nonSupineSleepDuration) if nonSupineSleepDuration > 0 else 0,
            "ahiRem": ((
                float(allValues["APNOEA_REM_NUMBER"]) + 
                float(allValues["HYPOPNOEA_REM_NUMBER"])
                ) / remDuration) if remDuration > 0 else 0,
            "ahiNrem": ((
                float(allValues["APNOEA_NONREM_NUMBER"]) + 
                float(allValues["HYPOPNOEA_NONREM_NUMBER"])
                ) / nonRemDuration) if nonRemDuration > 0 else 0,
            "percentageApneaCentral":
                (float(allValues["CENTRALE_SLEEP_NUMBER"]) / 
                apneaHyponea * 100) if apneaHyponea > 0 else 0,
            "rdiRuecken": ((
                float(allValues["APNOEA_SUPINE_NUMBER"]) + 
                float(allValues["HYPOPNOEA_SUPINE_NUMBER"]) +
                float(allValues["RERA_SUPINE_NUMBER"])
                ) / supineSleepDuration) if supineSleepDuration > 0 else 0,
            "rdiKeinRuecken": ((
                float(allValues["APNOEA_PRONE_NUMBER"]) + 
                float(allValues["APNOEA_LEFT_NUMBER"]) +
                float(allValues["APNOEA_RIGHT_NUMBER"]) +
                float(allValues["HYPOPNOEA_PRONE_NUMBER"]) +
                float(allValues["HYPOPNOEA_LEFT_NUMBER"]) +
                float(allValues["HYPOPNOEA_RIGHT_NUMBER"]) +
                float(allValues["RERA_PRONE_NUMBER"]) +
                float(allValues["RERA_LEFT_NUMBER"]) +
                float(allValues["RERA_RIGHT_NUMBER"])
                ) / nonSupineSleepDuration) if nonSupineSleepDuration > 0 else 0,
            "rdiRem": ((
                float(allValues["APNOEA_REM_NUMBER"]) + 
                float(allValues["HYPOPNOEA_REM_NUMBER"]) +
                float(allValues["RERA_REM_NUMBER"])
                ) / remDuration) if remDuration > 0 else 0,
            "rdiNrem": ((
                float(allValues["APNOEA_NONREM_NUMBER"]) + 
                float(allValues["HYPOPNOEA_NONREM_NUMBER"]) +
                float(allValues["RERA_NONREM_NUMBER"])
                ) / nonRemDuration) if nonRemDuration > 0 else 0,
            # "percentageSpO2Lower80": ((
            #     self.getTimeValue(allValues["TIME_90_SPO2"], unit="m") + 
            #     self.getTimeValue(allValues["TIME_90_SPO2_WAKE"], unit="m")
            #     ) / tibDuration) if tibDuration > 0 else 0,
        }
        data.update({
            "ratioAhiRuecken": (data["ahiRuecken"] / data["ahiKeinRuecken"]) if data["ahiKeinRuecken"] > 0 else 0,
            "ratioAhiRem": (data["ahiRem"] / data["ahiNrem"]) if data["ahiNrem"] > 0 else 0,
            "ratioRdiRuecken": (data["rdiRuecken"] / data["rdiKeinRuecken"]) if data["rdiKeinRuecken"] > 0 else 0,
            "ratioRdiRem": (data["rdiRem"] / data["rdiNrem"]) if data["rdiNrem"] > 0 else 0,
        })
        return data
    
    def getMetaDataFromFile(self, reportfile, rowIds):
        data = {}
        extracedIds = []
        with open(reportfile, newline="", encoding=DominoErgLoader.txtEncoding) as csvfile:
            rows = csv.reader(csvfile, delimiter="=")
            allData = {}
            for row in rows:
                # row = row.
                if len(row) < 2:
                    continue
                id, value = row[0:2]
                value = value.split(";")[-1]
                allData[id] = value
                if id in rowIds:
                    extracedIds.append(id)
                    if value != "":
                        nameId = rowIds[id]
                        if rowIds[id] == "":
                            nameId = "domino%s" % (id)

                        # check for duration
                        if ":" in value:
                            value = self.getTimeValue(value)
                        try:
                            if isinstance(value, str) and (id in ["PATIENTDATA_CASE_ID", "PATIENTDATA_ID", "PATIENTDATA_MONTAGE_NAME"]):
                                data[nameId] = value
                            # data.append((nameId, float(value), ""))
                            else:
                                data[nameId] = float(value)
                        except Exception as ex:
                            self.logWarning("Failed to convert folling Line to float %s:%s value: %s: %s" % (reportfile, id, value, ex))
        notFound = set(rowIds) - set(extracedIds)
        if len(notFound) > 0:
            self.logWarning("export of following ids not found: %s" % notFound)
        data.update(self.calculateMetaData(allData))
        return data

    # relevant rows where exported from the report files
    relevantRows = {
        "SLEEP_DURATION": "tst",
        "TIB_DURATION": "tib",
        "SLEEP_PERIODE_TOTAL": "spt",
        "SLEEPLATENCY": "sLatency",
        "REM_LATENCY": "rLatency",
        "WAKE_AFTER_SLEEP_ON": "waso",
        "SLEEP_EFFICIENCY": "sEfficiency",
        "WAKE_PERCENTAGE": "percentageW",
        "STAGE1_PERCENTAGE": "percentageN1",
        "STAGE2_PERCENTAGE": "percentageN2",
        "STAGE3_PERCENTAGE": "percentageN3",
        "REM_PERCENTAGE": "percentageR",
        "MA_TOTAL_SLEEP_NUMBER": "countArousal",
        "MA_TOTAL_SLEEP_INDEX": "indexArousal",
        "AVERAGE_HF": "meanHrSleep",
        "MAX_HF": "maxHrSleep",
        "MIN_HF": "minHrSleep",
        "PLM_SLEEP_NUMBER": "countPlms",
        "MA_PLM_SLEEP_NUMBER": "countPlmsArousal",
        "PLM_SLEEP_INDEX": "indexPlms",
        "MA_PLM_SLEEP_INDEX": "indexPlmsArousal",
        "OBSTRUCTIVE_SLEEP_NUMBER": "countApneaObstructive",
        "MIXED_SLEEP_NUMBER": "countApneaMixed",
        "CENTRALE_SLEEP_NUMBER": "countApneaCentral",
        "HYPOPNOEA_SLEEP_NUMBER": "countHypopnea",
        "APNOEA_SLEEP_INDEX": "indexApnea",
        "HYPOPNOEA_SLEEP_INDEX": "indexHypopnea",
        "AHI_SLEEP": "ahi",
        "RERA_SLEEP_NUMBER": "countRera",
        "RERA_SLEEP_INDEX": "indexRera",
        "RDI_SLEEP": "rdi",
        "NUMBER_DESAT": "countOxyDesat",
        "DESAT_INDEX": "indexOxyDesat",
        "AVERAGE_SPO2": "meanSpO2",
        "MIN_SPO2": "minSpO2",
        "AVERAGE_SPO2_REM": "meanSpO2Rem",
        "AVERAGE_SPO2_NONREM": "meanSpO2Nrem",
        "SNORE_TOTAL_EPISOD": "percentageSnore",
        "PATIENTDATA_ID": "patient",
        "PATIENTDATA_CASE_ID": "case",
        "PATIENTDATA_BMI": "bmi",
        "PATIENTDATA_MONTAGE_NAME": "psg_setup",
        "SLEEP_PROFILE_MANUALLY_SCORED": "manualScored",
    }



from pathlib import Path

from pyPhasesRecordloader import AnnotationNotFound
from pyPhasesRecordloader.recordLoaders.EDFRecordLoader import EDFRecordLoader

from .DominoAnnotationLoader import DominoAnnotationLoader
from .DominoErgLoader import DominoErgLoader


class RecordLoaderDomino(EDFRecordLoader):
    possibleNamesSleepstages = ["SchlafProfil", "Schlafprofil"]
    possibleRespEvents = ["Flow Events"]
    possibleArousalEvents = ["Klassifizierte Arousal"]

    def getFileBasePath(self, recrdId):
        return self.filePath + "/" + recrdId

    def getFilePathSignal(self, recordId):
        return self.getFileBasePath(recordId) + "/" + recordId + ".edf"

    def getFilePathAnnotation(self, recordId):
        annotationFolder = self.dataHandlerConfig.get("annotationFolder", "annotations")
        return self.getFileBasePath(recordId) + f"/{annotationFolder}"

    def getErgPath(self, recordId):
        return self.getFileBasePath(recordId) + "/" + recordId + ".erg"

    def existAnnotation(self, recordId):
        return [Path(self.getFilePathAnnotation(recordId) + f"/{n}.txt").exists() for n in self.possibleNamesSleepstages]

    def loadAnnotation(self, recordId, fileNames, valueMap=None, possibleValues=None):
        found = False
        for fileName in fileNames:
            filePath = self.getFilePathAnnotation(recordId) + "/" + fileName + ".txt"
            if Path(filePath).exists():
                found = True
                break

        if not found:
            raise AnnotationNotFound(fileName)

        annotationLoader = DominoAnnotationLoader.load(filePath, valueMap, self.annotationFrequency, possibleValues)

        return annotationLoader.events

    def getMetaData(self, recordName):
        metaData = super().getMetaData(recordName)
        # metaData.update(self.getAliceLoader().getMetaData(self.getFilePathAnnotation(recordName)))
        metaData.update(DominoErgLoader().getMetaDataFromFile(self.getErgPath(recordName), DominoErgLoader.relevantRows))

        # lightOff = self.getXMLPath(self.metaXML, ["Acquisition", "Sessions", "Session", "LightsOff"])
        # off, on = self.getLightAnnotations()
        # metaData["lightOff"] = off
        # metaData["lightOn"] = on

        return metaData

    def getLightAnnotations(self):
        """get light off, on in seconds"""
        lightOff = self.getLastAnnotationTimeByName("Licht aus")
        lightOn = self.getFirstAnnotationTimeByName("Ende der Messung")

        if lightOff is None:
            self.logError("Die Annotation 'Licht Aus' wurde nicht gefunden")
        else:
            lightOff = int(lightOff)
        if lightOn is None:
            self.logWarning("Die Annotation 'Ende der Messung' wurde nicht gefunden")
        else:
            lightOn = int(lightOn)

        return lightOff, lightOn

    def getEventList(self, recordName, targetFrequency=1):
        self.annotationFrequency = targetFrequency

        eventArray = self.loadAnnotation(
            recordName,
            self.possibleNamesSleepstages,
            {
                "N3": "N3",
                "N2": "N2",
                "N1": "N1",
                "Rem": "R",
                "Wach": "W",
                "Artefakt": "undefined",
            },
        )

        # eventArray = self.loadAnnotation(
        #     recordName,
        #     "Autonome Arousal",
        #     {
        #         "Autonome Arousal": "arousal",
        #     },
        # )
        try:
            eventArray += self.loadAnnotation(
                recordName,
                self.possibleRespEvents,
                {
                    "Obstruktive Apnoe": "resp_obstructiveapnea",
                    "Gemischte Apnoe": "resp_mixedapnea",
                    "Zentrale Apnoe": "resp_centralapnea",
                    "Hypopnoe": "resp_hypopnea",
                    "RERA": "resp_rera",
                },
                possibleValues=["Obstruktive Apnoe", "Gemischte Apnoe", "Zentrale Apnoe", "Hypopnoe", "RERA"],
            )
        except AnnotationNotFound:
            pass

        try:
            eventArray += self.loadAnnotation(
                recordName,
                self.possibleArousalEvents,
                {
                    "Arousal": "arousal",
                    "Respiratorische Arousal": "arousal_rera",
                    "PLM Arousal": "arousal_plm",
                    "LM Arousal": "arousal_limb",
                    "Schnarchen Arousal": "arousal_snore",
                    "SpO2 Arousal": "arousal",
                    "Arousal (EEG)": "arousal",
                    "Respiratorische Arousal (EEG)": "arousal_rera",
                    "PLM Arousal (EEG)": "arousal_plm",
                    "LM Arousal (EEG)": "arousal_limb",
                    "Schnarchen Arousal (EEG)": "arousal_snore",
                    "SpO2 Arousal (EEG)": "arousal",
                    "Arousal (Autonome)": "arousal",
                    "Respiratorische Arousal (Autonome)": "arousal_rera",
                    "PLM Arousal (Autonome)": "arousal_plm",
                    "LM Arousal (Autonome)": "arousal_limb",
                    "Schnarchen Arousal (Autonome)": "arousal_snore",
                    "SpO2 Arousal (Autonome)": "arousal",
                },
                possibleValues=[
                    "Arousal",
                    "Respiratorische Arousal",
                    "PLM Arousal",
                    "LM Arousal",
                    "Schnarchen Arousal",
                    "SpO2 Arousal",
                    "Arousal (EEG)",
                    "Respiratorische Arousal (EEG)",
                    "PLM Arousal (EEG)",
                    "LM Arousal (EEG)",
                    "Schnarchen Arousal (EEG)",
                    "SpO2 Arousal (EEG)",
                    "Arousal (Autonome)",
                    "Respiratorische Arousal (Autonome)",
                    "PLM Arousal (Autonome)",
                    "LM Arousal (Autonome)",
                    "Schnarchen Arousal (Autonome)",
                    "SpO2 Arousal (Autonome)",
                ],
            )
        except AnnotationNotFound:
            pass

        self.lightOff, self.lightOn = self.getLightAnnotations()

        return eventArray



import io
from dataclasses import dataclass
from datetime import timedelta

from pyPhases.util.Logger import classLogger

from teleschlafmedizin.model.recordManager.RecordMeta import RecordAnnotation

from ..RecordManager import RecordWriter
from .TSMRecordManager import Annotation, TSMRecordManager


@dataclass
class DominoAnnotation(Annotation):
    valueMapIndex = {}

    def __init__(self, id="", unit="", rate=None, signalType="Discret", fileName="", **kwargs):
        self.id = id
        self.unit = unit
        self.rate = rate
        self.signalType = signalType
        self.fileName = fileName
        super().__init__(**kwargs)

    @staticmethod
    def createAnnotation(name):
        a = Annotation.createAnnotation(name, DominoAnnotation())

        if name == "NeuroAdultAASMStaging":
            # a.possibleValues = ["N3", "N2", "N1", "Rem", "Wach"]
            a.fileName = "Schlafprofil"
            a.id = "SchlafProfil\profil"
            a.signalType = "Discret"
            a.rate = "30 s"
            a.valueMap = {
                "N3": "N3",
                "N2": "N2",
                "N1": "N1",
                "Rem": "R",
                "Wach": "W",
                "Artefakt": "undefined",
            }
            a.valueMapIndex = {
                "N3": 0,
                "N2": 1,
                "N1": 2,
                "R": 3,
                "W": 4,
                "undefined": 5,
            }
        elif name == "RespiratoryEvents":
            # a.possibleValues = ["N3", "N2", "N1", "Rem", "Wach"]
            a.fileName = "Flow Analyse"
            a.id = "FlowD\flow"
            a.signalType = "Impuls"
            a.unit = "s"
            a.valueMap = {
                "Obstruktive Apnoe": "resp_obstructiveapnea",
                "Hypopnoe": "resp_hypopnea",
                "Zentrale Apnoe": "resp_centralapnea",
                "Gemischte Apnoe": "resp_mixedapnea",
                "Obstruktive Hypopnoe": "resp_hypopnea_obstructive",
                "RERA": "resp_rera",
                "": "none",
            }
            a.valueMapIndex = {
                "resp_hypopnea": 1,
                "resp_obstructiveapnea": 2,
                "resp_centralapnea": 3,
                "resp_mixedapnea": 4,
                "resp_hypopnea_obstructive": 7,
                "resp_hypopnea_central": 8,
                "resp_rera": 9,
                "undefined": 5,
            }
            # Apnoe = 0
            # Flußlimitationen = 5
            # Körperlage Event = 6
        else:
            raise Exception("Domino export for %s not supported at the moment" % (name))

        #     signalType="Impuls",
        #     id="KorrelationMA\MAK",
        #     fileName="Klassifikation Arousal",
        #     possibleValues={
        #         "Undefined",
        #         "Respiratorisches Arousal",
        #         "PLM Arousal",
        #         "SpO2 Arousal",
        #         "Kardio Arousal",
        #         "Schnarch Arousal",
        #         "Arousal (EEG)",
        #         "Respiratorisches Arousal (EEG)",
        #         "PLM Arousal (EEG)",
        #         "SpO2 Arousal (EEG)",
        #         "Kardio Arousal (EEG)",
        #         "Schnarch Arousal (EEG)",
        #         "Arousal (EMG)",
        #         "Respiratorisches Arousal (EMG)",
        #         "PLM Arousal (EMG)",
        #         "SpO2 Arousal (EMG)",
        #         "Kardio Arousal (EMG)",
        #         "Schnarch Arousal (EMG)",
        #         "Flusslimitation Arousal",
        #         "Flusslimitation Arousal (EEG)",
        #         "Flusslimitation Arousal (EMG)",
        #         "RERA Arousal",
        #         "LM Arousal",
        #         "LM Arousal (EEG)",
        #         "LM Arousal (EMG)",
        #         "Arousal (Autonom)",
        #         "Respiratorisches Arousal (Autonom)",
        #         "PLM Arousal (Autonom",
        #         "SpO2 Arousal (Autonom)",
        #         "Kardio Arousal (Autonom)",
        #         "Schnarch Arousal (Autonom)",
        #         "Flusslimitation Arousal (Autonom)",
        #         "LM Arousal (Autonom)",
        #         "RERA Arousal (EEG)",
        #         "RERA Arousal (EMG)",
        #         "RERA Arousal (Autonom)",
        #     },

        # Spindel Analyse - Spindel\spindel
        #   Spindel = 0
        #   K-Komplex = 1

        # EOG Analyse - EOG\REM
        #   EM = 0
        #   link = 1

        # Schnarch-Analyse - Snore\snore
        #   Schnarchen = 0

        # SpO2 Analyse - Spo2\SpO2_Analyse
        #   Entsättigung = 0
        #   Körperlage Event = 1

        # LM Analyse - PLM\plm
        #   I-Marker = 0
        #   P-Marker = 1
        #   LM Resp = 2
        #   LM Körperlage = 3

        # Hertzfrequenzanalyse DOMINO - Kardio\Kardio_Analyse
        #   Akzeleration = 1
        #   Dezeleration = 2
        #   Arrhythmie = 3
        #   Tachykardie (breiter QRS Komplex) = 8
        #   Tachykardie (schmaler QRS Komplex) = 9
        #   Asystole = 10

        # Hertzfrequenzanalyse  DOMINO light - Kardio\Kardio_Analyse
        #   Akzeleration = 1
        #   Dezeleration = 2
        #   Arrhythmie = 3
        #   Tachykardie (breiter QRS Komplex) = 7
        #   Tachykardie (schmaler QRS Komplex) = 8
        #   Asystole = 9

        # Hertzfrequenz Analyse AASM DOMINO - KardioUSA\Kardio_Analyse
        #   Tachykardie (breiter QRS-Komplex) = 1
        #   Tachykardie (schmaler QRS Komplex) = 2
        #   Asystole = 3
        #   Arrhythmie = 6

        # Generic Analyse - Generic\generic
        #   Event1 = 0
        #   Event2 = 1
        #   Event3 = 2

        return a

    # def getBaseAnnotation(self) -> "Annotation":
    #     nameMap = {
    #         "Schlafprofil": "NeuroAdultAASMStaging",
    #         "Flow Analyse": "Apnea",
    #     }
    #     name = nameMap[self.name]
    #     baseAnnotation = Annotation.createAnnotation(name)
    #     baseAnnotation.values = self.values

    #     return baseAnnotation

    @staticmethod
    def fromBaseAnnotation(a) -> "DominoAnnotation":
        da = DominoAnnotation.createAnnotation(a.name)
        da.values = a.values

        return da

    @classmethod
    def fromDataAnnotation(cls, dataAnnotation: RecordAnnotation) -> "DominoAnnotation":
        baseAnnotation = Annotation.fromDataAnnotation(dataAnnotation)
        return cls.fromBaseAnnotation(baseAnnotation)


@classLogger
class DominoAnnotationWriter(RecordWriter, TSMRecordManager):

    lineEnding = "\n"
    annotation = DominoAnnotation

    def writerRecord(recordName):
        raise Exception("only Annotations are supported for Domino at the moment")

    def __init__(self) -> None:
        TSMRecordManager.__init__(self)
        self.xml = None
        self.patient = None

    def saveAnnotation(self, path, filename=""):
        for name, lines in self.annotationFiles.items():
            filename = "%s/%s.txt" % (path, name)
            with io.open(filename, "w", encoding="utf8") as f:
                f.write(self.lineEnding.join(lines) + self.lineEnding)

    def loadBaseAnnotationFile(self):
        self.annotationFiles = {}

        self.fileLines = []

    def addHeader(self, field, value):
        line = "%s: %s" % (field, value)
        self.fileLines.append(line)

    def addValue(self, value, time):
        line = "%s,000;%s" % (time, value)
        self.fileLines.append(line)

    def writeAnnotation(self, annotation: DominoAnnotation):
        self.fileLines = []

        self.addHeader("Signal ID", annotation.id)
        self.addHeader("Start Time", "19.11.2020 19:30:30")
        self.addHeader("Unit", annotation.unit)
        self.addHeader("Signal Type", annotation.signalType)
        self.addHeader("Events list", ",".join(annotation.possibleValues))

        if annotation.rate is not None:
            self.addHeader("Rate", annotation.rate)

        self.fileLines.append("")

        for ev in annotation.values:
            if ev.name != annotation.ignoreValue:
                valueIndex = annotation.valueMapIndex[ev.name]
                time = self.record.start + timedelta(seconds=ev.start)
                self.addValue(valueIndex, time)

        self.annotationFiles[annotation.fileName] = self.fileLines



from pyPhases import PluginAdapter
from pyPhasesRecordloader import RecordLoader


class Plugin(PluginAdapter):
    defaultConfig = "config.yaml"

    def initPlugin(self):
        # self.project.loadConfig(self.project.loadConfig(pathlib.Path(__file__).parent.absolute().joinpath("config.yaml")))
        module = "pyPhasesRecordloaderDomino"
        rlPath = f"{module}.recordLoaders"
        RecordLoader.registerRecordLoader("RecordLoaderDomino", rlPath)
        RecordLoader.registerRecordLoader("DominoAnnotationLoader", rlPath)
        dominoPath = self.getConfig("domino-path")
        self.project.setConfig("loader.domino.filePath", dominoPath)
